import pygame
from spw import canvas



class Context(object):
    current = None
    def __init__(self, screen):
        self.screen = screen
        self.canvas = canvas.Canvas(self.screen)
        self.dirty = []
        self.cleanup_fn = None
        Context.current = self

    def __enter__(self):
        self.canvas.clear()
        if self.cleanup_fn:
            self.cleanup_fn(self.dirty)
        else:
            for r in self.dirty:
                self.screen.fill((0,0,0), r)
        return self.canvas

    def __exit__(self, type, value, traceback):
        dirty = self.canvas._rects.keys()
        pygame.display.update(self.dirty + dirty)
        self.dirty = dirty



class Events(object):
    stack = []

    @classmethod
    def handle(self, event):
        for h in reversed(self.stack):
            if h.handle(event) is True: break

    @classmethod
    def capture(self, handler):
        assert hasattr(handler, "handle")
        self.stack.append(handler)
    
    @classmethod
    def release(self, handler):
        assert handler == self.stack[-1]
        self.stack.pop(-1)

    @classmethod 
    def loop(self):
        while True:
            e = pygame.event.wait()
            Events.handle(e)
            if e.type == pygame.QUIT: return
        
